@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2009  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2014  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -340,6 +340,14 @@ int console_charavail(int handle)
 		ssize_t len;
 		int i, d;
 
+		// If adding more would lead to a full FIFO, then let's
+		// wait.
+		int roomLeftInFIFO = console_handles[handle].fifo_tail - console_handles[handle].fifo_head;
+		if (roomLeftInFIFO <= 0)
+			roomLeftInFIFO += CONSOLE_FIFO_LEN;
+		if (roomLeftInFIFO < (int)sizeof(ch) + 1)
+			break;
+
 		if (!allow_slaves)
 			d = STDIN_FILENO;
 		else
