@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2009  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2018  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -194,7 +194,7 @@ uint8_t condition_gt[16] = { 1,0,1,0, 0,0,0,0, 0,1,0,1, 0,0,0,0 };
 		arm_instr_ ## n ## __hi, arm_instr_ ## n ## __ls,	\
 		arm_instr_ ## n ## __ge, arm_instr_ ## n ## __lt,	\
 		arm_instr_ ## n ## __gt, arm_instr_ ## n ## __le,	\
-		arm_instr_ ## n , arm_instr_nop };
+		arm_instr_ ## n , arm_instr_never };
 
 #define cond_instr(n)	( arm_cond_instr_ ## n  [condition_code] )
 
@@ -215,13 +215,31 @@ X(invalid) {
 
 	fatal("FATAL ERROR: An internal error occured in the ARM"
 	    " dyntrans code. Please contact the author with detailed"
-	    " repro steps on how to trigger this bug. pc = 0x%08"PRIx32"\n",
+	    " repro steps on how to trigger this bug. pc = 0x%08" PRIx32"\n",
 	    (uint32_t)cpu->pc);
 
 	cpu->cd.arm.next_ic = &nothing_call;
 }
 
 
+/*
+ *  never:  So far unimplemented "never" instructions end up here.
+ *  (Those are the ones using the "0xf" condition prefix.)
+ */
+X(never) {
+	uint32_t low_pc;
+	low_pc = ((size_t)ic - (size_t)
+	    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
+	cpu->pc &= ~((ARM_IC_ENTRIES_PER_PAGE-1)
+	    << ARM_INSTR_ALIGNMENT_SHIFT);
+	cpu->pc += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT);
+
+	fatal("[ ARM: unimplemented 0xf instruction at pc = 0x%08" PRIx32" ]\n", (uint32_t)cpu->pc);
+
+	cpu->cd.arm.next_ic = &nothing_call;
+}
+
+
 /*
  *  nop:  Do nothing.
  */
@@ -240,7 +258,7 @@ X(b)
 	cpu->pc = (uint32_t)((cpu->pc & 0xfffff000) + (int32_t)ic->arg[0]);
 
 	/*  Find the new physical page and update the translation pointers:  */
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 }
 Y(b)
 
@@ -334,15 +352,26 @@ void (*arm_cond_instr_b_samepage[16])(struct cpu *,
  */
 X(bx)
 {
+	uint32_t old_cpsr = cpu->cd.arm.cpsr;
 	cpu->pc = reg(ic->arg[0]);
-	if (cpu->pc & 1) {
-		fatal("thumb: TODO\n");
-		exit(1);
+	if (cpu->pc & 1)
+		cpu->cd.arm.cpsr |= ARM_FLAG_T;
+	else
+		cpu->cd.arm.cpsr &= ~ARM_FLAG_T;
+
+	if (cpu->cd.arm.cpsr != old_cpsr)
+		cpu->cd.arm.next_ic = &nothing_call;
+
+	if (cpu->pc & 2 && ((cpu->pc & 1) == 0)) {
+		fatal("[ ARM pc misaligned? 0x%08x ]\n", (int)cpu->pc);
+		cpu->running = 0;
+		cpu->n_translated_instrs --;
+		cpu->cd.arm.next_ic = &nothing_call;
+		return;
 	}
-	cpu->pc &= ~3;
 
 	/*  Find the new physical page and update the translation pointers:  */
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 }
 Y(bx)
 
@@ -354,17 +383,28 @@ Y(bx)
  */
 X(bx_trace)
 {
+	uint32_t old_cpsr = cpu->cd.arm.cpsr;
 	cpu->pc = cpu->cd.arm.r[ARM_LR];
-	if (cpu->pc & 1) {
-		fatal("thumb: TODO\n");
-		exit(1);
+	if (cpu->pc & 1)
+		cpu->cd.arm.cpsr |= ARM_FLAG_T;
+	else
+		cpu->cd.arm.cpsr &= ~ARM_FLAG_T;
+
+	if (cpu->cd.arm.cpsr != old_cpsr)
+		cpu->cd.arm.next_ic = &nothing_call;
+
+	if (cpu->pc & 2 && ((cpu->pc & 1) == 0)) {
+		fatal("[ ARM pc misaligned? 0x%08x ]\n", (int)cpu->pc);
+		cpu->running = 0;
+		cpu->n_translated_instrs --;
+		cpu->cd.arm.next_ic = &nothing_call;
+		return;
 	}
-	cpu->pc &= ~3;
 
 	cpu_functioncall_trace_return(cpu);
 
 	/*  Find the new physical page and update the translation pointers:  */
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 }
 Y(bx_trace)
 
@@ -373,6 +413,7 @@ Y(bx_trace)
  *  bl:  Branch and Link (to a different translated page)
  *
  *  arg[0] = relative address
+ *  arg[1] = offset within current page to the instruction
  */
 X(bl)
 {
@@ -383,31 +424,88 @@ X(bl)
 	cpu->pc = pc + (int32_t)ic->arg[0];
 
 	/*  Find the new physical page and update the translation pointers:  */
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 }
 Y(bl)
 
 
 /*
- *  blx:  Branch and Link, potentially exchanging Thumb/ARM encoding
+ *  blx_imm:  Branch and Link, always switching to THUMB encoding
+ *
+ *  arg[0] = relative address
+ *  arg[1] = offset within current page to the current instruction
+ */
+X(blx_imm)
+{
+	uint32_t pc = ((uint32_t)cpu->pc & 0xfffff000) + (int32_t)ic->arg[1];
+	uint32_t old_cpsr = cpu->cd.arm.cpsr;
+	cpu->cd.arm.r[ARM_LR] = pc + 4;
+
+	/*  Calculate new PC from this instruction + arg[0]  */
+	cpu->pc = pc + (int32_t)ic->arg[0];
+
+	if (cpu->pc & 1)
+		cpu->cd.arm.cpsr |= ARM_FLAG_T;
+	else {
+		fatal("[ blx_imm internal error. Should have switched to THUMB! 0x%08x ]\n", (int)cpu->pc);
+		cpu->running = 0;
+		cpu->n_translated_instrs --;
+		cpu->cd.arm.next_ic = &nothing_call;
+		return;
+	}
+
+	if (cpu->cd.arm.cpsr != old_cpsr)
+		cpu->cd.arm.next_ic = &nothing_call;
+
+	if (cpu->pc & 2 && ((cpu->pc & 1) == 0)) {
+		fatal("[ ARM pc misaligned? 0x%08x ]\n", (int)cpu->pc);
+		cpu->running = 0;
+		cpu->n_translated_instrs --;
+		cpu->cd.arm.next_ic = &nothing_call;
+		return;
+	}
+
+	if (cpu->machine->show_trace_tree)
+		cpu_functioncall_trace(cpu, cpu->pc);
+
+	/*  Find the new physical page and update the translation pointers:  */
+	quick_pc_to_pointers_arm(cpu);
+}
+
+
+/*
+ *  blx_reg:  Branch and Link, potentially exchanging Thumb/ARM encoding
  *
  *  arg[0] = ptr to rm
+ *  arg[2] = offset within current page to the instruction to return to
  */
-X(blx)
+X(blx_reg)
 {
 	uint32_t lr = ((uint32_t)cpu->pc & 0xfffff000) + (int32_t)ic->arg[2];
 	cpu->cd.arm.r[ARM_LR] = lr;
 	cpu->pc = reg(ic->arg[0]);
-	if (cpu->pc & 1) {
-		fatal("thumb: TODO\n");
-		exit(1);
+
+	uint32_t old_cpsr = cpu->cd.arm.cpsr;
+	if (cpu->pc & 1)
+		cpu->cd.arm.cpsr |= ARM_FLAG_T;
+	else
+		cpu->cd.arm.cpsr &= ~ARM_FLAG_T;
+
+	if (cpu->cd.arm.cpsr != old_cpsr)
+		cpu->cd.arm.next_ic = &nothing_call;
+
+	if (cpu->pc & 2 && ((cpu->pc & 1) == 0)) {
+		fatal("[ ARM pc misaligned? 0x%08x ]\n", (int)cpu->pc);
+		cpu->running = 0;
+		cpu->n_translated_instrs --;
+		cpu->cd.arm.next_ic = &nothing_call;
+		return;
 	}
-	cpu->pc &= ~3;
 
 	/*  Find the new physical page and update the translation pointers:  */
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 }
-Y(blx)
+Y(blx_reg)
 
 
 /*
@@ -426,7 +524,7 @@ X(bl_trace)
 	cpu_functioncall_trace(cpu, cpu->pc);
 
 	/*  Find the new physical page and update the translation pointers:  */
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 }
 Y(bl_trace)
 
@@ -679,14 +777,14 @@ X(ret_trace)
 		    ((cpu->pc & mask_within_page) >> ARM_INSTR_ALIGNMENT_SHIFT);
 	} else {
 		/*  Find the new physical page and update pointers:  */
-		quick_pc_to_pointers(cpu);
+		quick_pc_to_pointers_arm(cpu);
 	}
 }
 Y(ret_trace)
 X(ret)
 {
 	cpu->pc = cpu->cd.arm.r[ARM_LR];
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 }
 Y(ret)
 
@@ -704,6 +802,11 @@ Y(ret)
 X(msr_imm)
 {
 	uint32_t mask = ic->arg[1];
+
+	if ((cpu->cd.arm.cpsr & ARM_FLAG_MODE) == ARM_MODE_USR32) {
+		mask &= 0xff000000;
+	}
+
 	int switch_register_banks = (mask & ARM_FLAG_MODE) &&
 	    ((cpu->cd.arm.cpsr & ARM_FLAG_MODE) !=
 	    (ic->arg[0] & ARM_FLAG_MODE));
@@ -757,15 +860,15 @@ X(msr_imm_spsr)
 		break;
 	default:fatal("msr_spsr: unimplemented mode %i\n",
 		    cpu->cd.arm.cpsr & ARM_FLAG_MODE);
-{
-	/*  Synchronize the program counter:  */
-	uint32_t old_pc, low_pc = ((size_t)ic - (size_t)
-	    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
-	cpu->pc &= ~((ARM_IC_ENTRIES_PER_PAGE-1) << ARM_INSTR_ALIGNMENT_SHIFT);
-	cpu->pc += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT);
-	old_pc = cpu->pc;
-	printf("msr_spsr: old pc = 0x%08"PRIx32"\n", old_pc);
-}
+		{
+			/*  Synchronize the program counter:  */
+			uint32_t old_pc, low_pc = ((size_t)ic - (size_t)
+			    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
+			cpu->pc &= ~((ARM_IC_ENTRIES_PER_PAGE-1) << ARM_INSTR_ALIGNMENT_SHIFT);
+			cpu->pc += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT);
+			old_pc = cpu->pc;
+			printf("msr_spsr: old pc = 0x%08" PRIx32"\n", old_pc);
+		}
 		exit(1);
 	}
 }
@@ -849,7 +952,7 @@ X(openfirmware)
 	cpu->pc = cpu->cd.arm.r[ARM_LR];
 	if (cpu->machine->show_trace_tree)
 		cpu_functioncall_trace_return(cpu);
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 }
 
 
@@ -1008,36 +1111,16 @@ X(cmps_regshort);
 
 
 /*
- *  bdt_load:  Block Data Transfer, Load
- *
- *  arg[0] = pointer to uint32_t in host memory, pointing to the base register
- *  arg[1] = 32-bit instruction word. Most bits are read from this.
+ *  Shared between regular ARM and the THUMB encoded 'pop'.
  */
-X(bdt_load)
+void arm_pop(struct cpu* cpu, uint32_t* np, int p_bit, int u_bit, int s_bit, int w_bit, uint32_t iw)
 {
+	uint32_t addr = *np;
 	unsigned char data[4];
-	uint32_t *np = (uint32_t *)ic->arg[0];
-	uint32_t addr = *np, low_pc;
 	unsigned char *page;
-	uint32_t iw = ic->arg[1];  /*  xxxx100P USWLnnnn llllllll llllllll  */
-	int p_bit = iw & 0x01000000;
-	int u_bit = iw & 0x00800000;
-	int s_bit = iw & 0x00400000;
-	int w_bit = iw & 0x00200000;
 	int i, return_flag = 0;
 	uint32_t new_values[16];
 
-#ifdef GATHER_BDT_STATISTICS
-	if (!s_bit)
-		update_bdt_statistics(iw);
-#endif
-
-	/*  Synchronize the program counter:  */
-	low_pc = ((size_t)ic - (size_t)
-	    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
-	cpu->pc &= ~((ARM_IC_ENTRIES_PER_PAGE-1) << ARM_INSTR_ALIGNMENT_SHIFT);
-	cpu->pc += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT);
-
 	if (s_bit) {
 		/*  Load to USR registers:  */
 		if ((cpu->cd.arm.cpsr & ARM_FLAG_MODE) == ARM_MODE_USR32) {
@@ -1188,44 +1271,23 @@ X(bdt_load)
 		    same page!  */
 		/*  Find the new physical page and update the
 		    translation pointers:  */
-		quick_pc_to_pointers(cpu);
+		quick_pc_to_pointers_arm(cpu);
 	}
 }
-Y(bdt_load)
 
 
 /*
- *  bdt_store:  Block Data Transfer, Store
- *
- *  arg[0] = pointer to uint32_t in host memory, pointing to the base register
- *  arg[1] = 32-bit instruction word. Most bits are read from this.
+ *  Shared between regular ARM and the THUMB encoded 'push'.
  */
-X(bdt_store)
+void arm_push(struct cpu* cpu, uint32_t* np, int p_bit, int u_bit, int s_bit, int w_bit, uint16_t regs)
 {
-	unsigned char data[4];
-	uint32_t *np = (uint32_t *)ic->arg[0];
-	uint32_t low_pc, value, addr = *np;
-	uint32_t iw = ic->arg[1];  /*  xxxx100P USWLnnnn llllllll llllllll  */
-	unsigned char *page;
-	int p_bit = iw & 0x01000000;
-	int u_bit = iw & 0x00800000;
-	int s_bit = iw & 0x00400000;
-	int w_bit = iw & 0x00200000;
 	int i;
-
-#ifdef GATHER_BDT_STATISTICS
-	if (!s_bit)
-		update_bdt_statistics(iw);
-#endif
-
-	/*  Synchronize the program counter:  */
-	low_pc = ((size_t)ic - (size_t)
-	    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
-	cpu->pc &= ~((ARM_IC_ENTRIES_PER_PAGE-1) << ARM_INSTR_ALIGNMENT_SHIFT);
-	cpu->pc += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT);
+	uint32_t value, addr = *np;
+	unsigned char data[4];
+	unsigned char *page;
 
 	for (i=(u_bit? 0 : 15); i>=0 && i<=15; i+=(u_bit? 1 : -1)) {
-		if (!((iw >> i) & 1)) {
+		if (!((regs >> i) & 1)) {
 			/*  Skip register i:  */
 			continue;
 		}
@@ -1307,6 +1369,69 @@ X(bdt_store)
 	if (w_bit)
 		*np = addr;
 }
+
+
+/*
+ *  bdt_load:  Block Data Transfer, Load
+ *
+ *  arg[0] = pointer to uint32_t in host memory, pointing to the base register
+ *  arg[1] = 32-bit instruction word. Most bits are read from this.
+ */
+X(bdt_load)
+{
+	uint32_t *np = (uint32_t *)ic->arg[0];
+	uint32_t low_pc;
+	uint32_t iw = ic->arg[1];  /*  xxxx100P USWLnnnn llllllll llllllll  */
+	int p_bit = iw & 0x01000000;
+	int u_bit = iw & 0x00800000;
+	int s_bit = iw & 0x00400000;
+	int w_bit = iw & 0x00200000;
+
+#ifdef GATHER_BDT_STATISTICS
+	if (!s_bit)
+		update_bdt_statistics(iw);
+#endif
+
+	/*  Synchronize the program counter:  */
+	low_pc = ((size_t)ic - (size_t)
+	    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
+	cpu->pc &= ~((ARM_IC_ENTRIES_PER_PAGE-1) << ARM_INSTR_ALIGNMENT_SHIFT);
+	cpu->pc += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT);
+
+	arm_pop(cpu, np, p_bit, u_bit, s_bit, w_bit, (uint16_t)iw);
+}
+Y(bdt_load)
+
+
+/*
+ *  bdt_store:  Block Data Transfer, Store
+ *
+ *  arg[0] = pointer to uint32_t in host memory, pointing to the base register
+ *  arg[1] = 32-bit instruction word. Most bits are read from this.
+ */
+X(bdt_store)
+{
+	uint32_t *np = (uint32_t *)ic->arg[0];
+	uint32_t low_pc;
+	uint32_t iw = ic->arg[1];  /*  xxxx100P USWLnnnn llllllll llllllll  */
+	int p_bit = iw & 0x01000000;
+	int u_bit = iw & 0x00800000;
+	int s_bit = iw & 0x00400000;
+	int w_bit = iw & 0x00200000;
+
+#ifdef GATHER_BDT_STATISTICS
+	if (!s_bit)
+		update_bdt_statistics(iw);
+#endif
+
+	/*  Synchronize the program counter:  */
+	low_pc = ((size_t)ic - (size_t)
+	    cpu->cd.arm.cur_ic_page) / sizeof(struct arm_instr_call);
+	cpu->pc &= ~((ARM_IC_ENTRIES_PER_PAGE-1) << ARM_INSTR_ALIGNMENT_SHIFT);
+	cpu->pc += (low_pc << ARM_INSTR_ALIGNMENT_SHIFT);
+
+	arm_push(cpu, np, p_bit, u_bit, s_bit, w_bit, (uint16_t)iw);
+}
 Y(bdt_store)
 
 
@@ -1753,7 +1878,7 @@ X(cmps_0_beq)
 		cpu->cd.arm.flags = ARM_F_Z | ARM_F_C;
 		cpu->pc = (uint32_t)(((uint32_t)cpu->pc & 0xfffff000)
 		    + (int32_t)ic[1].arg[0]);
-		quick_pc_to_pointers(cpu);
+		quick_pc_to_pointers_arm(cpu);
 	} else {
 		/*  Semi-ugly hack which sets the negative-bit if a < 0:  */
 		cpu->cd.arm.flags = ARM_F_C | ((a >> 28) & 8);
@@ -1771,7 +1896,7 @@ X(cmps_pos_beq)
 		cpu->cd.arm.flags |= ARM_F_Z;
 		cpu->pc = (uint32_t)(((uint32_t)cpu->pc & 0xfffff000)
 		    + (int32_t)ic[1].arg[0]);
-		quick_pc_to_pointers(cpu);
+		quick_pc_to_pointers_arm(cpu);
 	} else {
 		cpu->cd.arm.next_ic = &ic[2];
 		if (c & 0x80000000)
@@ -1789,7 +1914,7 @@ X(cmps_neg_beq)
 		cpu->cd.arm.flags |= ARM_F_Z;
 		cpu->pc = (uint32_t)(((uint32_t)cpu->pc & 0xfffff000)
 		    + (int32_t)ic[1].arg[0]);
-		quick_pc_to_pointers(cpu);
+		quick_pc_to_pointers_arm(cpu);
 	} else {
 		cpu->cd.arm.next_ic = &ic[2];
 		if (c & 0x80000000)
@@ -2062,7 +2187,7 @@ X(end_of_page)
 	cpu->pc += (ARM_IC_ENTRIES_PER_PAGE << ARM_INSTR_ALIGNMENT_SHIFT);
 
 	/*  Find the new physical page and update the translation pointers:  */
-	quick_pc_to_pointers(cpu);
+	quick_pc_to_pointers_arm(cpu);
 
 	/*  end_of_page doesn't count as an executed instruction:  */
 	cpu->n_translated_instrs --;
@@ -2501,7 +2626,7 @@ X(to_be_translated)
 	unsigned char *page;
 	unsigned char ib[4];
 	int condition_code, main_opcode, secondary_opcode, s_bit, rn, rd, r8;
-	int p_bit, u_bit, w_bit, l_bit, regform, rm, c, t, any_pc_reg;
+	int p_bit, u_bit, w_bit, l_bit, regform, rm, any_pc_reg; // , c, t
 	void (*samepage_function)(struct cpu *, struct arm_instr_call *);
 
 	/*  Figure out the address of the instruction:  */
@@ -2551,28 +2676,45 @@ X(to_be_translated)
 	rn    = (iword >> 16) & 15;
 	rd    = (iword >> 12) & 15;
 	r8    = (iword >> 8) & 15;
-	c     = (iword >> 7) & 31;
-	t     = (iword >> 4) & 7;
+	// c     = (iword >> 7) & 31;
+	// t     = (iword >> 4) & 7;
 	rm    = iword & 15;
 
-	if (condition_code == 0xf) {
+	/*
+	 *  Translate the instruction:
+	 */
+
+	if ((iword >> 28) == 0xf) {
+		/*  The "never" condition is nowadays used for special encodings.  */
 		if ((iword & 0xfc70f000) == 0xf450f000) {
 			/*  Preload:  TODO.  Treat as NOP for now.  */
 			ic->f = instr(nop);
 			goto okay;
 		}
 
-		if (!cpu->translation_readahead)
-			fatal("TODO: ARM condition code 0x%x\n",
-			    condition_code);
-		goto bad;
+		switch (main_opcode) {
+		case 0xa:
+		case 0xb:
+			ic->f = instr(blx_imm);
+
+			/*  arg 1 = offset of current instruction  */
+			ic->arg[1] = addr & 0xffc;
+
+			/*  arg 0 = relative jump distance + 1 (to enable THUMB)  */
+			ic->arg[0] = (iword & 0x00ffffff) << 2;
+			/*  Sign-extend:  */
+			if (ic->arg[0] & 0x02000000)
+				ic->arg[0] |= 0xfc000000;
+			if (main_opcode == 0xb)
+				ic->arg[0] += 2;
+			ic->arg[0] = (int32_t)(ic->arg[0] + 8 + 1);
+			break;
+		default:
+			goto bad;
+		}
+		goto okay;
 	}
 
-
-	/*
-	 *  Translate the instruction:
-	 */
-
 	switch (main_opcode) {
 
 	case 0x0:
@@ -2622,7 +2764,7 @@ X(to_be_translated)
 		if ((iword & 0x0ff000d0) == 0x01200010) {
 			/*  bx or blx  */
 			if (iword & 0x20)
-				ic->f = cond_instr(blx);
+				ic->f = cond_instr(blx_reg);
 			else {
 				if (cpu->machine->show_trace_tree &&
 				    rm == ARM_LR)
@@ -2631,6 +2773,7 @@ X(to_be_translated)
 					ic->f = cond_instr(bx);
 			}
 			ic->arg[0] = (size_t)(&cpu->cd.arm.r[rm]);
+			ic->arg[2] = (addr & 0xffc) + 4;
                         break;
                 }
 		if ((iword & 0x0fb00ff0) == 0x1000090) {
@@ -2703,13 +2846,18 @@ X(to_be_translated)
 				imm = (imm >> 2) | ((imm & 3) << 30);
 			ic->arg[0] = imm;
 			ic->arg[2] = (size_t)(&cpu->cd.arm.r[rm]);
-			switch ((iword >> 16) & 15) {
-			case 1:	ic->arg[1] = 0x000000ff; break;
-			case 8:	ic->arg[1] = 0xff000000; break;
-			case 9:	ic->arg[1] = 0xff0000ff; break;
-			default:if (!cpu->translation_readahead)
-					fatal("unimpl a: msr regform\n");
-				goto bad;
+			{
+				uint32_t arg1 = 0;
+				if (iword & (1<<16)) arg1 |= 0x000000ff;
+				if (iword & (1<<17)) arg1 |= 0x0000ff00;
+				if (iword & (1<<18)) arg1 |= 0x00ff0000;
+				if (iword & (1<<19)) arg1 |= 0xff000000;
+				if (arg1 == 0) {
+					if (!cpu->translation_readahead)
+						fatal("msr no fields\n");
+					goto bad;
+				}
+				ic->arg[1] = arg1;
 			}
 			break;
 		}
@@ -2728,8 +2876,8 @@ X(to_be_translated)
 			break;
 		}
 		if ((iword & 0x0e000090) == 0x00000090) {
-			int imm = ((iword >> 4) & 0xf0) | (iword & 0xf);
-			int regform = !(iword & 0x00400000);
+			regform = !(iword & 0x00400000);
+			imm = ((iword >> 4) & 0xf0) | (iword & 0xf);
 			p_bit = main_opcode & 1;
 			ic->arg[0] = (size_t)(&cpu->cd.arm.r[rn]);
 			ic->arg[2] = (size_t)(&cpu->cd.arm.r[rd]);
@@ -2837,9 +2985,19 @@ X(to_be_translated)
 				q = 0;
 			ic->arg[1] = (size_t)(void *)arm_r[(iword & 0xfff) + q];
 		} else {
+			int steps = r8;
+
 			imm = iword & 0xff;
+			
 			while (r8-- > 0)
 				imm = (imm >> 2) | ((imm & 3) << 30);
+
+			if (steps != 0 && (imm != 0 && imm < 256)) {
+				if (!cpu->translation_readahead)
+					fatal("TODO: see cpu_arm_instr_dpi; non-zero steps but still under 256 is not implemented yet\n");
+				goto bad;
+			}
+
 			ic->arg[1] = imm;
 		}
 
@@ -2911,25 +3069,26 @@ X(to_be_translated)
 			/*  NOTE/TODO: This assumes 4KB pages,
 			    it will not work with 1KB pages.  */
 			if (ofs >= 0 && ofs <= max && p != NULL) {
-				unsigned char c[4];
+				unsigned char cbuf[4];
 				int len = b_bit? 1 : 4;
 				uint32_t x, a = (addr & 0xfffff000) | ofs;
 				/*  ic->f = cond_instr(mov);  */
 				ic->f = arm_dpi_instr[condition_code + 16*0xd];
 				ic->arg[2] = (size_t)(&cpu->cd.arm.r[rd]);
 
-				memcpy(c, p + (a & 0xfff), len);
+				memcpy(cbuf, p + (a & 0xfff), len);
 
 				if (b_bit) {
-					x = c[0];
+					x = cbuf[0];
 				} else {
 					if (cpu->byte_order == EMUL_LITTLE_ENDIAN)
-						x = c[0] + (c[1]<<8) +
-						    (c[2]<<16) + (c[3]<<24);
+						x = cbuf[0] + (cbuf[1]<<8) +
+						    (cbuf[2]<<16) + (cbuf[3]<<24);
 					else
-						x = c[3] + (c[2]<<8) +
-						    (c[1]<<16) + (c[0]<<24);
+						x = cbuf[3] + (cbuf[2]<<8) +
+						    (cbuf[1]<<16) + (cbuf[0]<<24);
 				}
+				
 				ic->arg[1] = x;
 			}
 		}
