@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2009  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2007-2018  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -42,7 +42,9 @@
                 cpu->pc += (low_pc << M88K_INSTR_ALIGNMENT_SHIFT);      \
         }
 
-#define	ABORT_EXECUTION	  {	cpu->cd.m88k.next_ic = &nothing_call;	\
+#define	ABORT_EXECUTION	  {	SYNCH_PC;				\
+				fatal("Execution aborted at: pc = 0x%08x\n", (int)cpu->pc); \
+				cpu->cd.m88k.next_ic = &nothing_call;	\
 				cpu->running = 0;			\
 				debugger_n_steps_left_before_interaction = 0; }
 
@@ -848,7 +850,7 @@ X(fadd_sss)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_S);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_S);
 
-	d = ieee_store_float_value(f1.f + f2.f, IEEE_FMT_S, 0);
+	d = ieee_store_float_value(f1.f + f2.f, IEEE_FMT_S);
 
 	reg(ic->arg[0]) = d;
 }
@@ -871,7 +873,7 @@ X(fadd_dsd)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_S);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_D);
 
-	d = ieee_store_float_value(f1.f + f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f + f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -895,7 +897,7 @@ X(fadd_dds)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_D);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_S);
 
-	d = ieee_store_float_value(f1.f + f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f + f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -920,7 +922,7 @@ X(fadd_ddd)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_D);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_D);
 
-	d = ieee_store_float_value(f1.f + f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f + f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -944,7 +946,7 @@ X(fsub_sds)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_D);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_S);
 
-	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_S, 0);
+	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_S);
 	reg(ic->arg[0]) = d;
 }
 X(fsub_dss)
@@ -965,7 +967,7 @@ X(fsub_dss)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_S);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_S);
 
-	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -989,7 +991,7 @@ X(fsub_dsd)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_S);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_D);
 
-	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1013,7 +1015,7 @@ X(fsub_dds)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_D);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_S);
 
-	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1038,7 +1040,7 @@ X(fsub_ddd)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_D);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_D);
 
-	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f - f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1061,7 +1063,7 @@ X(fmul_sss)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_S);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_S);
 
-	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_S, 0);
+	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_S);
 	reg(ic->arg[0]) = d;
 }
 X(fmul_dss)
@@ -1082,7 +1084,7 @@ X(fmul_dss)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_S);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_S);
 
-	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1106,7 +1108,7 @@ X(fmul_dsd)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_S);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_D);
 
-	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1130,7 +1132,7 @@ X(fmul_dds)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_D);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_S);
 
-	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1155,7 +1157,7 @@ X(fmul_ddd)
 	ieee_interpret_float_value(s1, &f1, IEEE_FMT_D);
 	ieee_interpret_float_value(s2, &f2, IEEE_FMT_D);
 
-	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f * f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1185,7 +1187,7 @@ X(fdiv_sss)
 		return;
 	}
 
-	d = ieee_store_float_value(f1.f / f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f / f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d;
 }
@@ -1215,7 +1217,7 @@ X(fdiv_dsd)
 		return;
 	}
 
-	d = ieee_store_float_value(f1.f / f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f / f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1247,7 +1249,7 @@ X(fdiv_ddd)
 		return;
 	}
 
-	d = ieee_store_float_value(f1.f / f2.f, IEEE_FMT_D, 0);
+	d = ieee_store_float_value(f1.f / f2.f, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = d >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = d;	/*  and low word.  */
@@ -1350,7 +1352,7 @@ X(flt_ss)
 		return;
 	}
 
-	reg(ic->arg[0]) = ieee_store_float_value((double)x, IEEE_FMT_S, 0);
+	reg(ic->arg[0]) = ieee_store_float_value((double)x, IEEE_FMT_S);
 }
 X(flt_ds)
 {
@@ -1364,7 +1366,7 @@ X(flt_ds)
 		return;
 	}
 
-	result = ieee_store_float_value((double)x, IEEE_FMT_D, 0);
+	result = ieee_store_float_value((double)x, IEEE_FMT_D);
 
 	reg(ic->arg[0]) = result >> 32;	/*  High 32-bit word,  */
 	reg(ic->arg[0] + 4) = result;	/*  and low word.  */
@@ -1513,14 +1515,14 @@ X(rte)
 		return;
 	}
 
-	/*  fatal("RTE: NIP=0x%08"PRIx32", FIP=0x%08"PRIx32"\n",
+	/*  fatal("RTE: NIP=0x%08" PRIx32", FIP=0x%08" PRIx32"\n",
 	    cpu->cd.m88k.cr[M88K_CR_SNIP], cpu->cd.m88k.cr[M88K_CR_SFIP]);  */
 
 	quick_pc_to_pointers(cpu);
 	return;
 
 abort_dump:
-	fatal("RTE failed. NIP=0x%08"PRIx32", FIP=0x%08"PRIx32"\n",
+	fatal("RTE failed. NIP=0x%08" PRIx32", FIP=0x%08" PRIx32"\n",
 	    cpu->cd.m88k.cr[M88K_CR_SNIP], cpu->cd.m88k.cr[M88K_CR_SFIP]);
 
 	ABORT_EXECUTION;
@@ -1635,9 +1637,15 @@ X(prom_call)
 	}
 
 	switch (cpu->machine->machine_type) {
+
+	case MACHINE_LUNA88K:
+		luna88kprom_emul(cpu);
+		break;
+
 	case MACHINE_MVME88K:
 		mvmeprom_emul(cpu);
 		break;
+
 	default:fatal("m88k prom_call: unimplemented machine type\n");
 		ABORT_EXECUTION;
 	}
@@ -1908,7 +1916,7 @@ X(to_be_translated)
 	unsigned char *page;
 	unsigned char ib[4];
 	uint32_t op26, op10, op11, d, s1, s2, cr6, imm16;
-	int32_t d16, d26, simm16;
+	int32_t d16, d26; //, simm16;
 	int offset, shift;
 	int in_crosspage_delayslot = 0;
 	void (*samepage_function)(struct cpu *, struct m88k_instr_call *)=NULL;
@@ -1981,7 +1989,7 @@ X(to_be_translated)
 	s1     = (iword >> 16) & 0x1f;
 	s2     =  iword        & 0x1f;
 	imm16  =  iword        & 0xffff;
-	simm16 = (int16_t) (iword & 0xffff);
+	// simm16 = (int16_t) (iword & 0xffff);
 	cr6    = (iword >>  5) & 0x3f;
 	d16    = ((int16_t) (iword & 0xffff)) * 4;
 	d26    = ((int32_t)((iword & 0x03ffffff) << 6)) >> 4;
@@ -2697,7 +2705,7 @@ X(to_be_translated)
 				if (iword == 0xf400fc00)
 					ic->f = instr(rte);
 				else {
-					fatal("unimplemented rte variant: 0x%08"PRIx32"\n", iword);
+					fatal("unimplemented rte variant: 0x%08" PRIx32"\n", iword);
 					goto bad;
 				}
 				break;
