@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2006-2009  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2006-2014  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -58,7 +58,7 @@
 #include "thirdparty/dreamcast_sysasicvar.h"
 
 
-#define debug fatal
+// #define debug fatal
 
 #define	N_MAPLE_PORTS		4
 
@@ -94,25 +94,23 @@ struct dreamcast_maple_data {
  *
  *  TODO: Figure out strings and numbers of _real_ Maple devices.
  */
-#if 0
-static struct maple_device maple_device_controller = {
+struct maple_device maple_device_controller = {
 	{
 		BE32_TO_HOST(MAPLE_FUNC(MAPLE_FN_CONTROLLER)),	/*  di_func  */
-		{ 0,0,0 },			/*  di_function_data[3]  */
-		0,				/*  di_area_code  */
+		{ LE32_TO_HOST(0xfe060f00),0,0 },/* di_function_data[3]  */
+		0xff,				/*  di_area_code  */
 		0,				/*  di_connector_direction  */
 		"Dreamcast Controller",		/*  di_product_name  */
 		"di_product_license",		/*  di_product_license  */
-		LE16_TO_HOST(100),		/*  di_standby_power  */
-		LE16_TO_HOST(100)		/*  di_max_power  */
+		LE16_TO_HOST(0x01ae),		/*  di_standby_power  */
+		LE16_TO_HOST(0x01f4)		/*  di_max_power  */
 	}
 };
-#endif
-static struct maple_device maple_device_keyboard = {
+struct maple_device maple_device_keyboard = {
 	{
 		BE32_TO_HOST(MAPLE_FUNC(MAPLE_FN_KEYBOARD)),/*  di_func  */
 		{ LE32_TO_HOST(2),0,0 },	/*  di_function_data[3]  */
-		0,				/*  di_area_code  */
+		0xff,				/*  di_area_code  */
 		0,				/*  di_connector_direction  */
 		"Keyboard",			/*  di_product_name  */
 		"di_product_license",		/*  di_product_license  */
@@ -120,20 +118,18 @@ static struct maple_device maple_device_keyboard = {
 		LE16_TO_HOST(100)		/*  di_max_power  */
 	}
 };
-#if 0
-static struct maple_device maple_device_mouse = {
+struct maple_device maple_device_mouse = {
 	{
 		BE32_TO_HOST(MAPLE_FUNC(MAPLE_FN_MOUSE)),/*  di_func  */
-		{ 0,0,0 },			/*  di_function_data[3]  */
-		0,				/*  di_area_code  */
+		{ LE32_TO_HOST(0xfe060f00),0,0 },/* di_function_data[3]  */
+		0xff,				/*  di_area_code  */
 		0,				/*  di_connector_direction  */
 		"Dreamcast Mouse",		/*  di_product_name  */
 		"di_product_license",		/*  di_product_license  */
-		LE16_TO_HOST(100),		/*  di_standby_power  */
-		LE16_TO_HOST(100)		/*  di_max_power  */
+		LE16_TO_HOST(0x0069),		/*  di_standby_power  */
+		LE16_TO_HOST(0x0120)		/*  di_max_power  */
 	}
 };
-#endif
 
 
 DEVICE_TICK(maple)
@@ -147,72 +143,80 @@ DEVICE_TICK(maple)
 		d->char_queue_head = (d->char_queue_head + 1) % MAX_CHARS;
 		if (d->char_queue_head == d->char_queue_tail)
 			fatal("[ dreamcast_maple: KEYBOARD QUEUE OVERRUN! ]\n");
-	}
 
-#if 0
-	/*
-	 *  NOTE/TODO:
-	 *
-	 *  Implement the controller in a reasonable way!
-	 */
+		/*
+		 *  NOTE/TODO:
+		 *
+		 *  Implement the controller in a reasonable way!
+		 *  Theoretically, there are all of these buttons: ABCD XYZ Start
+		 *  and two digital pads, two analog joysticks, and two
+		 *  analog fire buttons (left and right).
+		 */
 
-	control_bits = 0;
-	switch (key) {
-	case 'a':
-	case 'A':
-		control_bits = 0x0004;
-		break;
-	case 'b':
-	case 'B':
-		control_bits = 0x0002;
-		break;
-	case 'c':
-	case 'C':
-		control_bits = 0x0001;
-		break;
-	case 'x':
-	case 'X':
-		control_bits = 0x0400;
-		break;
-	case 'y':
-	case 'Y':
-		control_bits = 0x0200;
-		break;
-	case 'z':
-	case 'Z':
-		control_bits = 0x0100;
-		break;
-	case 's':
-	case 'S':	/*  Start  */
-		control_bits = 0x0008;
-		break;
-	case '8':	/*  up  */
-		control_bits = 0x0010;
-		break;
-	case '2':	/*  down  */
-	case 'k':
-		control_bits = 0x0020;
-		break;
-	case '4':	/*  left  */
-	case 'u':
-		control_bits = 0x0040;
-		break;
-	case '6':	/*  right  */
-	case 'o':
-		control_bits = 0x0080;
-		break;
-	}
+		int control_bits = 0;
+		switch (key) {
+		case 'a':
+		case 'A':
+			control_bits = 0x0004;
+			break;
+		case 'b':
+		case 'B':
+			control_bits = 0x0002;
+			break;
+		case 'c':
+		case 'C':
+			control_bits = 0x0001;
+			break;
+		case 'd':
+		case 'D':
+			control_bits = 0x0800;
+			break;
 
-	if (control_bits != 0) {
-		/*  Add to the controller queue:  */
-		d->controller_queue[d->controller_queue_head] = control_bits;
-		d->controller_queue_head =
-		    (d->controller_queue_head + 1) % MAX_CONTROLLER_DATA;
-		if (d->controller_queue_head == d->controller_queue_tail)
-			fatal("[ dreamcast_maple: CONTROLLER QUEUE "
-			    "OVERRUN! ]\n");
+		case 'x':
+		case 'X':
+			control_bits = 0x0400;
+			break;
+		case 'y':
+		case 'Y':
+			control_bits = 0x0200;
+			break;
+		case 'z':
+		case 'Z':
+			control_bits = 0x0100;
+			break;
+
+		case 's':
+		case 'S':	/*  Start  */
+			control_bits = 0x0008;
+			break;
+
+		case '8':	/*  up  */
+			control_bits = 0x0010;
+			break;
+		case '2':	/*  down  */
+		case 'k':
+			control_bits = 0x0020;
+			break;
+		case '4':	/*  left  */
+		case 'u':
+			control_bits = 0x0040;
+			break;
+		case '6':	/*  right  */
+		case 'o':
+			control_bits = 0x0080;
+			break;
+		}
+
+		if (control_bits != 0) {
+			/*  Add to the controller queue:  */
+			d->controller_queue[d->controller_queue_head] = control_bits;
+			d->controller_queue_head =
+			    (d->controller_queue_head + 1) % MAX_CONTROLLER_DATA;
+			if (d->controller_queue_head == d->controller_queue_tail)
+				fatal("[ dreamcast_maple: CONTROLLER QUEUE "
+				    "OVERRUN! ]\n");
+		}
 	}
-#endif
 }
 
 
@@ -281,7 +285,8 @@ static void maple_getcond_controller_response(struct dreamcast_maple_data *d,
 	memset(buf, 0, 8);
 	buf[0] = c & 0xff;
 	buf[1] = c >> 8;
-	buf[4] = buf[5] = buf[6] = buf[7] = 0;
+	buf[2] = buf[3] = 0;				// 0 = not pressed
+	buf[4] = buf[5] = buf[6] = buf[7] = 128;	// 128 = centered
 
 	cpu->memory_rw(cpu, cpu->mem, receive_addr, (unsigned char *) (void *) &buf, 8,
 	    MEM_WRITE, NO_EXCEPTIONS | PHYSICAL);
@@ -377,19 +382,19 @@ void maple_do_dma_xfer(struct cpu *cpu, struct dreamcast_maple_data *d)
 	uint32_t addr = d->dmaaddr;
 
 	if (!d->enable) {
-		fatal("[ maple_do_dma_xfer: not enabled? ]\n");
+		debug("[ maple_do_dma_xfer: not enabled? ]\n");
 		return;
 	}
 
 	/*  debug("[ dreamcast_maple: DMA transfer, dmaaddr = "
-	    "0x%08"PRIx32" ]\n", addr);  */
+	    "0x%08" PRIx32" ]\n", addr);  */
 
 	/*
 	 *  DMA transfers must be 32-byte aligned, according to Marcus
 	 *   Comstedt's Maple demo program.
 	 */
 	if (addr & 0x1f) {
-		fatal("[ dreamcast_maple: dmaaddr 0x%08"PRIx32" is NOT"
+		fatal("[ dreamcast_maple: dmaaddr 0x%08" PRIx32" is NOT"
 		    " 32-byte aligned; aborting ]\n", addr);
 		return;
 	}
@@ -427,7 +432,7 @@ void maple_do_dma_xfer(struct cpu *cpu, struct dreamcast_maple_data *d)
 
 		if (receive_addr & 0xe000001f)
 			fatal("[ dreamcast_maple: WARNING! receive address 0x"
-			    "%08"PRIx32" isn't valid! ]\n", receive_addr);
+			    "%08" PRIx32" isn't valid! ]\n", receive_addr);
 
 		/*  Read the command word for this message:  */
 		cpu->memory_rw(cpu, cpu->mem, addr, (unsigned char *) (void *) &buf, 4, MEM_READ,
@@ -454,9 +459,9 @@ void maple_do_dma_xfer(struct cpu *cpu, struct dreamcast_maple_data *d)
 			exit(1);
 		}
 
-		/*  debug("[ dreamcast_maple: cmd=0x%02x, port=%c, unit=%i"
-		    ", datalen=%i words ]\n", cmd, port+'A', unit,
-		    datalen_cmd);  */
+		/*  fatal("[ dreamcast_maple: cmd=0x%02x, port=%c, unit=%i"
+		    ", datalen=%i words. receive_addr=0x%08x ]\n",
+		    cmd, port+'A', unit, datalen_cmd, receive_addr);  */
 
 		/*
 		 *  Handle the command:
@@ -510,7 +515,8 @@ void maple_do_dma_xfer(struct cpu *cpu, struct dreamcast_maple_data *d)
 				    d, cpu, port, receive_addr);
 			} else {
 				fatal("[ dreamcast_maple: WARNING: GETCOND: "
-				    "UNIMPLEMENTED 0x%08"PRIx32" ]\n", cond);
+				    "UNIMPLEMENTED 0x%08" PRIx32" ]\n", cond);
+				exit(1);
 			}
 			break;
 
@@ -529,6 +535,9 @@ void maple_do_dma_xfer(struct cpu *cpu, struct dreamcast_maple_data *d)
 			break;
 	}
 
+	// TODO: Should the DMA address be updated?
+	d->dmaaddr = addr;
+
 	/*  Assert the SYSASIC_EVENT_MAPLE_DMADONE event:  */
 	SYSASIC_TRIGGER_EVENT(SYSASIC_EVENT_MAPLE_DMADONE);
 }
@@ -547,11 +556,12 @@ DEVICE_ACCESS(dreamcast_maple)
 	case 0x04:	/*  MAPLE_DMAADDR  */
 		if (writeflag == MEM_WRITE) {
 			d->dmaaddr = idata;
-			/*  debug("[ dreamcast_maple: dmaaddr set to 0x%08x"
+			/* debug("[ dreamcast_maple: dmaaddr set to 0x%08x"
 			    " ]\n", d->dmaaddr);  */
 		} else {
 			fatal("[ dreamcast_maple: TODO: read from dmaaddr ]\n");
 			odata = d->dmaaddr;
+			exit(1);
 		}
 		break;
 
@@ -597,13 +607,23 @@ DEVICE_ACCESS(dreamcast_maple)
 
 	case 0x8c:	/*  MAPLE_RESET  */
 		if (writeflag == MEM_WRITE) {
-			if (idata != 0x6155404f)
+			if (idata != 0x6155404f && idata != 0x61557f00)
 				fatal("[ dreamcast_maple: UNIMPLEMENTED reset "
 				    "value 0x%08x ]\n", (int)idata);
 			d->enable = 0;
 		}
 		break;
 
+	case 0xe8:	/*  UNKNOWN_0xe8  */
+		if (writeflag == MEM_WRITE) {
+			// The Dreamcast PROM writes 1 here.
+			if (idata != 1) {
+				fatal("[ dreamcast_maple: UNIMPLEMENTED 0xe8 "
+				    "value 0x%08x ]\n", (int)idata);
+			}
+		}
+		break;
+
 	default:if (writeflag == MEM_READ) {
 			fatal("[ dreamcast_maple: UNIMPLEMENTED read from "
 			    "addr 0x%x ]\n", (int)relative_addr);
@@ -631,7 +651,7 @@ DEVINIT(dreamcast_maple)
 	    0x5f6c00, 0x100, dev_dreamcast_maple_access, d, DM_DEFAULT, NULL);
 
 	/*  Devices connected to port A..D:  */
-	d->device[0] = NULL;	/*  &maple_device_controller;  */
+	d->device[0] = &maple_device_controller;
 	d->device[1] = NULL;
 	d->device[2] = &maple_device_keyboard;
 	d->device[3] = NULL;	/*  TODO:  &maple_device_mouse;  */
