@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2006-2009  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2006-2018  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -29,10 +29,10 @@
  *
  *  TODO: Among other things:
  *
- *	x)  Interrupt masks (msk register stuff).
+ *	x)  Interrupt masks (msk register stuff). Are these really correct?
  *	x)  BSC (Bus state controller).
- *	x)  DMA
- *	x)  UBC
+ *	x)  DMA: Right now there's a hack for Dreamcast emulation
+ *	x)  UBC (User Break Controller)
  *	x)  ...
  */
 
@@ -79,18 +79,19 @@
 #define	SCIF_TX_FIFO_SIZE	16
 #define	SCIF_DELAYED_TX_VALUE	2	/*  2 to be safe, 1 = fast but buggy  */
 
-/*  General-purpose I/O stuff:  */
-#define	SH4_PCTRA		0xff80002c
-#define	SH4_PDTRA		0xff800030
-#define	SH4_PCTRB		0xff800040
-#define	SH4_PDTRB		0xff800044
-#define	SH4_GPIOIC		0xff800048
-
 #ifdef UNSTABLE_DEVEL
-#define SH4_DEGUG
+// #define SH4_DEBUG
 /*  #define debug fatal  */
 #endif
 
+// Clock/occilation related
+#define SH4_CPG_FRQCR		0xffc00000	/* 16-bit */
+#define SH4_CPG_STBCR 		0xffc00004	/* 8-bit */
+#define SH4_CPG_WTCNT 		0xffc00008	/* 8/16-bit */
+#define SH4_CPG_WTCSR 		0xffc0000c	/* 8/16-bit */
+#define SH4_CPG_STBCR2 		0xffc00010	/* 8-bit */
+
+
 struct sh4_data {
 	/*  Store Queues:  */
 	uint8_t		sq[32 * 2];
@@ -114,18 +115,29 @@ struct sh4_data {
 	/*  Bus State Controller:  */
 	uint32_t	bsc_bcr1;
 	uint16_t	bsc_bcr2;
+	uint16_t	bsc_bcr3;	/*  SH7751R  */
 	uint32_t	bsc_wcr1;
 	uint32_t	bsc_wcr2;
+	uint32_t	bsc_wcr3;
 	uint32_t	bsc_mcr;
+	uint16_t	bsc_pcr;
 	uint16_t	bsc_rtcsr;
 	uint16_t	bsc_rtcor;
 	uint16_t	bsc_rfcr;
 
+	/*  CPG:  */
+	uint16_t	cpg_frqcr;
+	uint8_t		cpg_stbcr;
+	uint16_t	cpg_wtcnt;
+	uint16_t	cpg_wtcsr;
+	uint8_t		cpg_stbcr2;
+
 	/*  GPIO:  */
 	uint32_t	pctra;		/*  Port Control Register A  */
 	uint32_t	pdtra;		/*  Port Data Register A  */
 	uint32_t	pctrb;		/*  Port Control Register B  */
 	uint32_t	pdtrb;		/*  Port Data Register B  */
+	uint16_t	bsc_gpioic;
 
 	/*  PCIC (PCI controller):  */
 	struct pci_data	*pci_data;
@@ -155,8 +167,9 @@ struct sh4_data {
 	double		timer_hz[N_SH4_TIMERS];
 
 	/*  RTC:  */
-	uint32_t	rtc_reg[14];	/*  Excluding rcr1 and 2  */
+	uint32_t	rtc_reg[14];	/*  Excluding rcr1 and rcr2  */
 	uint8_t		rtc_rcr1;
+	uint8_t		rtc_rcr2;
 };
 
 
@@ -362,7 +375,7 @@ void sh4_dmac_transfer(struct cpu *cpu, struct sh4_data *d, int channel)
 	src_delta *= transmit_size;
 	dst_delta *= transmit_size;
 
-#ifdef SH4_DEGUG
+#ifdef SH4_DEBUG
 	fatal("|SH4 DMA transfer, channel %i\n", channel);
 	fatal("|Source addr:      0x%08x (delta %i)\n", (int) sar, src_delta);
 	fatal("|Destination addr: 0x%08x (delta %i)\n", (int) dar, dst_delta);
@@ -378,6 +391,10 @@ void sh4_dmac_transfer(struct cpu *cpu, struct sh4_data *d, int channel)
 		 *  External Address Space => external device
 		 */
 
+		// Avoid compiler warnings about unused sar and dar.
+		(void)sar;
+		(void)dar;
+
 		/*  Note: No transfer is done here! It is up to the
 		    external device to do the transfer itself!  */
 		break;
@@ -828,7 +845,7 @@ DEVICE_ACCESS(sh4_pcic)
 		/*  Hardcoded to what OpenBSD/landisk uses:  */
 		if (writeflag == MEM_WRITE && idata != 0xac000000) {
 			fatal("sh4_pcic: SH4_PCICONF5 unknown value"
-			    " 0x%"PRIx32"\n", (uint32_t) idata);
+			    " 0x%" PRIx32"\n", (uint32_t) idata);
 			exit(1);
 		}
 		break;
@@ -837,7 +854,7 @@ DEVICE_ACCESS(sh4_pcic)
 		/*  Hardcoded to what OpenBSD/landisk uses:  */
 		if (writeflag == MEM_WRITE && idata != 0x8c000000) {
 			fatal("sh4_pcic: SH4_PCICONF6 unknown value"
-			    " 0x%"PRIx32"\n", (uint32_t) idata);
+			    " 0x%" PRIx32"\n", (uint32_t) idata);
 			exit(1);
 		}
 		break;
@@ -846,7 +863,7 @@ DEVICE_ACCESS(sh4_pcic)
 		/*  Hardcoded to what OpenBSD/landisk uses:  */
 		if (writeflag == MEM_WRITE && idata != ((64 - 1) << 20)) {
 			fatal("sh4_pcic: SH4_PCILSR0 unknown value"
-			    " 0x%"PRIx32"\n", (uint32_t) idata);
+			    " 0x%" PRIx32"\n", (uint32_t) idata);
 			exit(1);
 		}
 		break;
@@ -855,7 +872,7 @@ DEVICE_ACCESS(sh4_pcic)
 		/*  Hardcoded to what OpenBSD/landisk uses:  */
 		if (writeflag == MEM_WRITE && idata != 0xac000000) {
 			fatal("sh4_pcic: SH4_PCILAR0 unknown value"
-			    " 0x%"PRIx32"\n", (uint32_t) idata);
+			    " 0x%" PRIx32"\n", (uint32_t) idata);
 			exit(1);
 		}
 		break;
@@ -864,7 +881,7 @@ DEVICE_ACCESS(sh4_pcic)
 		/*  Hardcoded to what OpenBSD/landisk uses:  */
 		if (writeflag == MEM_WRITE && idata != ((64 - 1) << 20)) {
 			fatal("sh4_pcic: SH4_PCILSR1 unknown value"
-			    " 0x%"PRIx32"\n", (uint32_t) idata);
+			    " 0x%" PRIx32"\n", (uint32_t) idata);
 			exit(1);
 		}
 		break;
@@ -873,15 +890,15 @@ DEVICE_ACCESS(sh4_pcic)
 		/*  Hardcoded to what OpenBSD/landisk uses:  */
 		if (writeflag == MEM_WRITE && idata != 0xac000000) {
 			fatal("sh4_pcic: SH4_PCILAR1 unknown value"
-			    " 0x%"PRIx32"\n", (uint32_t) idata);
+			    " 0x%" PRIx32"\n", (uint32_t) idata);
 			exit(1);
 		}
 		break;
 
 	case SH4_PCIMBR:
 		if (writeflag == MEM_WRITE && idata != SH4_PCIC_MEM) {
-			fatal("sh4_pcic: PCIMBR set to 0x%"PRIx32", not"
-			    " 0x%"PRIx32"? TODO\n", (uint32_t) idata,
+			fatal("sh4_pcic: PCIMBR set to 0x%" PRIx32", not"
+			    " 0x%" PRIx32"? TODO\n", (uint32_t) idata,
 			    (uint32_t) SH4_PCIC_MEM);
 			exit(1);
 		}
@@ -889,8 +906,8 @@ DEVICE_ACCESS(sh4_pcic)
 
 	case SH4_PCIIOBR:
 		if (writeflag == MEM_WRITE && idata != SH4_PCIC_IO) {
-			fatal("sh4_pcic: PCIIOBR set to 0x%"PRIx32", not"
-			    " 0x%"PRIx32"? TODO\n", (uint32_t) idata,
+			fatal("sh4_pcic: PCIIOBR set to 0x%" PRIx32", not"
+			    " 0x%" PRIx32"? TODO\n", (uint32_t) idata,
 			    (uint32_t) SH4_PCIC_IO);
 			exit(1);
 		}
@@ -966,7 +983,7 @@ DEVICE_ACCESS(sh4)
 			d->sdmr3 = v;
 		else
 			d->sdmr2 = v;
-		debug("[ sh4: sdmr%i set to 0x%04"PRIx16" ]\n",
+		debug("[ sh4: sdmr%i set to 0x%04" PRIx16" ]\n",
 		    relative_addr & 0x00040000? 3 : 2, v);
 		return 1;
 	}
@@ -1107,7 +1124,18 @@ DEVICE_ACCESS(sh4)
 	/********************************/
 	/*  UBC: User Break Controller  */
 
-	case 0xff200008:    /*  SH4_BBRA  */
+	case 0xff000014:    /*  SH4_UBC_BASRA  */
+	case 0xff000018:    /*  SH4_UBC_BASRB  */
+
+	case 0xff200000:    /*  SH4_UBC_BARA  */
+	case 0xff200004:    /*  SH4_UBC_BAMRA  */
+	case 0xff200008:    /*  SH4_UBC_BBRA  */
+	case 0xff20000c:    /*  SH4_UBC_BARB  */
+	case 0xff200010:    /*  SH4_UBC_BAMRB  */
+	case 0xff200014:    /*  SH4_UBC_BBRB  */
+	case 0xff200018:    /*  SH4_UBC_BDRB  */
+	case 0xff20001c:    /*  SH4_UBC_BDMRB  */
+	case 0xff200020:    /*  SH4_UBC_BRCR  */
 		/*  TODO  */
 		break;
 
@@ -1208,7 +1236,7 @@ DEVICE_ACCESS(sh4)
 			if (idata & (TCR_ICPF | TCR_ICPE1 | TCR_ICPE0 |
 			    TCR_CKEG1 | TCR_CKEG0 | TCR_TPSC2)) {
 				fatal("Unimplemented SH4 timer control"
-				    " bits: 0x%08"PRIx32". Aborting.\n",
+				    " bits: 0x%08" PRIx32". Aborting.\n",
 				    (int) idata);
 				exit(1);
 			}
@@ -1237,7 +1265,7 @@ DEVICE_ACCESS(sh4)
 	case SH4_SAR3:	dma_channel ++;
 	case SH4_SAR2:	dma_channel ++;
 	case SH4_SAR1:	dma_channel ++;
-	case SH4_SAR0:	dma_channel ++;
+	case SH4_SAR0:
 		if (writeflag == MEM_READ)
 			odata = cpu->cd.sh.dmac_sar[dma_channel];
 		else
@@ -1251,7 +1279,7 @@ DEVICE_ACCESS(sh4)
 	case SH4_DAR3:	dma_channel ++;
 	case SH4_DAR2:	dma_channel ++;
 	case SH4_DAR1:	dma_channel ++;
-	case SH4_DAR0:	dma_channel ++;
+	case SH4_DAR0:
 		if (writeflag == MEM_READ)
 			odata = cpu->cd.sh.dmac_dar[dma_channel];
 		else
@@ -1265,7 +1293,7 @@ DEVICE_ACCESS(sh4)
 	case SH4_DMATCR3: dma_channel ++;
 	case SH4_DMATCR2: dma_channel ++;
 	case SH4_DMATCR1: dma_channel ++;
-	case SH4_DMATCR0: dma_channel ++;
+	case SH4_DMATCR0:
 		if (writeflag == MEM_READ)
 			odata = cpu->cd.sh.dmac_tcr[dma_channel] & 0x00ffffff;
 		else {
@@ -1287,7 +1315,7 @@ DEVICE_ACCESS(sh4)
 	case SH4_CHCR3:	dma_channel ++;
 	case SH4_CHCR2:	dma_channel ++;
 	case SH4_CHCR1:	dma_channel ++;
-	case SH4_CHCR0:	dma_channel ++;
+	case SH4_CHCR0:
 		if (writeflag == MEM_READ) {
 			odata = cpu->cd.sh.dmac_chcr[dma_channel];
 		} else {
@@ -1302,6 +1330,15 @@ DEVICE_ACCESS(sh4)
 		}
 		break;
 
+	case SH4_DMAOR:
+		if (writeflag == MEM_READ) {
+			odata = cpu->cd.sh.dmaor;
+		} else {
+			// Only some bits are writable:
+			idata &= (DMAOR_DDT | DMAOR_PR1 | DMAOR_PR0 | DMAOR_DME);
+			cpu->cd.sh.dmaor = idata;
+		}
+		break;
 
 	/*************************************************/
 	/*  BSC: Bus State Controller                    */
@@ -1327,6 +1364,17 @@ DEVICE_ACCESS(sh4)
 			odata = d->bsc_bcr2;
 		break;
 
+	case SH4_BCR3:
+		if (len != sizeof(uint16_t)) {
+			fatal("Non-16-bit SH4_BCR3 access?\n");
+			exit(1);
+		}
+		if (writeflag == MEM_WRITE)
+			d->bsc_bcr3 = idata;
+		else
+			odata = d->bsc_bcr3;
+		break;
+
 	case SH4_WCR1:
 		if (writeflag == MEM_WRITE)
 			d->bsc_wcr1 = idata & 0x77777777;
@@ -1341,6 +1389,13 @@ DEVICE_ACCESS(sh4)
 			odata = d->bsc_wcr2;
 		break;
 
+	case SH4_WCR3:
+		if (writeflag == MEM_WRITE)
+			d->bsc_wcr3 = idata & 0x77777777;
+		else
+			odata = d->bsc_wcr3;
+		break;
+
 	case SH4_MCR:
 		if (writeflag == MEM_WRITE)
 			d->bsc_mcr = idata & 0xf8bbffff;
@@ -1348,6 +1403,13 @@ DEVICE_ACCESS(sh4)
 			odata = d->bsc_mcr;
 		break;
 
+	case SH4_PCR:
+		if (writeflag == MEM_WRITE)
+			d->bsc_pcr = idata;
+		else
+			odata = d->bsc_pcr;
+		break;
+
 	case SH4_RTCSR:
 		/*
 		 *  Refresh Time Control/Status Register. Called RTCSR in
@@ -1385,20 +1447,35 @@ DEVICE_ACCESS(sh4)
 	/*  GPIO:  General-purpose I/O controller  */
 
 	case SH4_PCTRA:
-		if (writeflag == MEM_WRITE)
+		if (writeflag == MEM_WRITE) {
 			d->pctra = idata;
-		else
+			
+			// Hack: Makes the Dreamcast BIOS pass "cable select"
+			// detection, it seems, without hanging in an endless
+			// loop.
+			d->pdtra |= 0x03;
+		} else {
 			odata = d->pctra;
+		}
 		break;
 
 	case SH4_PDTRA:
 		if (writeflag == MEM_WRITE) {
-			debug("[ sh4: pdtra: write: TODO ]\n");
+			// debug("[ sh4: pdtra: write 0x%08x (while pctra = 0x%08x) ]\n", (int)idata, (int)d->pctra);
 			d->pdtra = idata;
+
+			// Hack: Makes the Dreamcast BIOS pass "cable select"
+			// detection, it seems, without hanging in an endless
+			// loop.
+			if ((idata & 1) == 0 || (idata & 2) == 0)
+				d->pdtra &= ~3;
 		} else {
-			debug("[ sh4: pdtra: read: TODO ]\n");
+			// debug("[ sh4: pdtra: read ]\n");
 			odata = d->pdtra;
-odata = random();
+
+			// bits 8..9 on Dreamcast mean:
+			//  00 = VGA, 10 = RGB, 11 = composite.
+			odata |= (0 << 8);
 		}
 		break;
 
@@ -1419,6 +1496,13 @@ odata = random();
 		}
 		break;
 
+	case SH4_GPIOIC:
+		if (writeflag == MEM_WRITE)
+			d->bsc_gpioic = idata;
+		else
+			odata = d->bsc_gpioic;
+		break;
+
 
 	/****************************/
 	/*  SCI:  Serial Interface  */
@@ -1629,15 +1713,58 @@ odata = random();
 		}
 		break;
 
+	case SH4_SCIF_BASE + SCIF_FDR:
+		/*  Nr of bytes in the TX and RX fifos, respectively:  */
+		odata = (console_charavail(d->scif_console_handle)? 1 : 0)
+		    + (d->scif_tx_fifo_cursize << 8);
+		break;
+
+	case SH4_SCIF_BASE + SCIF_SPTR:
+		/*  TODO: Implement all bits.  */
+		odata = 0;
+		break;
+
 	case SH4_SCIF_BASE + SCIF_LSR:
 		/*  TODO: Implement all bits.  */
 		odata = 0;
 		break;
 
-	case SH4_SCIF_BASE + SCIF_FDR:
-		/*  Nr of bytes in the TX and RX fifos, respectively:  */
-		odata = (console_charavail(d->scif_console_handle)? 1 : 0)
-		    + (d->scif_tx_fifo_cursize << 8);
+
+	/*************************************************/
+
+	case SH4_CPG_FRQCR:	// 0xffc00000	16-bit
+		if (writeflag == MEM_WRITE)
+			d->cpg_frqcr = idata;
+		else
+			odata = d->cpg_frqcr;
+		break;
+
+	case SH4_CPG_STBCR: 	// 0xffc00004	8-bit
+		if (writeflag == MEM_WRITE)
+			d->cpg_stbcr = idata;
+		else
+			odata = d->cpg_stbcr;
+		break;
+
+	case SH4_CPG_WTCNT:	// 0xffc00008	8/16-bit
+		if (writeflag == MEM_WRITE)
+			d->cpg_wtcnt = idata;
+		else
+			odata = d->cpg_wtcnt;
+		break;
+
+	case SH4_CPG_WTCSR:	// 0xffc0000c	8/16-bit
+		if (writeflag == MEM_WRITE)
+			d->cpg_wtcsr = idata;
+		else
+			odata = d->cpg_wtcsr;
+		break;
+
+	case SH4_CPG_STBCR2:	// 0xffc00010	8-bit
+		if (writeflag == MEM_WRITE)
+			d->cpg_stbcr2 = idata;
+		else
+			odata = d->cpg_stbcr2;
 		break;
 
 
@@ -1676,6 +1803,18 @@ odata = random();
 		}
 		break;
 
+	case SH4_RCR2:
+		if (writeflag == MEM_READ)
+			odata = d->rtc_rcr2;
+		else {
+			d->rtc_rcr2 = idata;
+			// bit 1 (i.e. idata == 0x02) means reset.
+			if (idata != 0x02) {
+				debug("[ SH4: TODO: RTC RCR2 value 0x%02x ignored. ]\n", (int)idata);
+			}
+		}
+		break;
+
 
 	/*************************************************/
 
@@ -1686,8 +1825,9 @@ odata = random();
 			fatal("[ sh4: write to addr 0x%x: 0x%x ]\n",
 			    (int)relative_addr, (int)idata);
 		}
-#ifdef SH4_DEGUG
-		/*  exit(1);  */
+
+#ifdef SH4_DEBUG
+		exit(1);
 #endif
 	}
 
@@ -1747,7 +1887,7 @@ DEVINIT(sh4)
 	 *  0xf4000000	SH4_CCDA	D-Cache address array
 	 *  0xf5000000	SH4_CCDD	D-Cache data array
 	 *
-	 *  TODO: Implement more correct cache behaviour?
+	 *  TODO: Implement more correct cache behavior?
 	 */
 
 	dev_ram_init(machine, SH4_CCIA, SH4_ICACHE_SIZE * 2, DEV_RAM_RAM, 0x0);
@@ -1855,6 +1995,11 @@ DEVINIT(sh4)
 	}
 
 
+	// The RTC RCR2 register is "basically" initialized to 0x09, with
+	// some bit undefined. :-) According to the manual.
+	d->rtc_rcr2 = 0x09;
+
+
 	/*
 	 *  Bus State Controller initial values, according to the
 	 *  SH7760 manual:
@@ -1863,7 +2008,7 @@ DEVINIT(sh4)
 	d->bsc_bcr2 = 0x3ffc;
 	d->bsc_wcr1 = 0x77777777;
 	d->bsc_wcr2 = 0xfffeefff;
-
+	d->bsc_wcr3 = 0x77777777;
 
 	return 1;
 }
