@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2009  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2018  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -226,12 +226,38 @@ int64_t diskimage_get_baseoffset(struct machine *machine, int id, int type)
 	while (d != NULL) {
 		if (d->type == type && d->id == id)
 			return d->override_base_offset;
+
 		d = d->next;
 	}
 	return -1;
 }
 
 
+/*
+ *  diskimage_set_baseoffset():
+ *
+ *  Sets the base offset for a disk image. Useful e.g. when booting directly
+ *  from NetBSD/dreamcast or Linux/dreamcast ISO images.
+ */
+void diskimage_set_baseoffset(struct machine *machine, int id, int type, int64_t offset)
+{
+	struct diskimage *d = machine->first_diskimage;
+
+	while (d != NULL) {
+		if (d->type == type && d->id == id) {
+			d->override_base_offset = offset;
+			return;
+		}
+		
+		d = d->next;
+	}
+
+	fatal("diskimage_set_baseoffset(): disk id %i (type %i) not found?\n",
+	    id, diskimage_types[type]);
+	exit(1);
+}
+
+
 /*
  *  diskimage_getchs():
  *
@@ -706,7 +732,7 @@ int diskimage_add(struct machine *machine, char *fname)
 				if (*fname == ':' || *fname == ';')
 					fname ++;
 				if (override_base_offset < 0) {
-					fatal("Bad base offset: %"PRIi64
+					fatal("Bad base offset: %" PRIi64
 					    "\n", override_base_offset);
 					exit(1);
 				}
@@ -743,6 +769,7 @@ int diskimage_add(struct machine *machine, char *fname)
 	/*  ... but some machines use SCSI by default:  */
 	if (machine->machine_type == MACHINE_PMAX ||
 	    machine->machine_type == MACHINE_ARC ||
+	    machine->machine_type == MACHINE_SGI ||
 	    machine->machine_type == MACHINE_MVME88K)
 		d->type = DISKIMAGE_SCSI;
 
@@ -884,8 +911,13 @@ int diskimage_add(struct machine *machine, char *fname)
 
 	d->writable = access(fname, W_OK) == 0? 1 : 0;
 
-	if (d->is_a_cdrom || prefix_r)
+	if (d->is_a_cdrom || prefix_r) {
 		d->writable = 0;
+	} else {
+		if (!d->writable) {
+			debug("NOTE: '%s' is read-only in the host file system, but 'r:' was not used.\n\n", d->fname);
+		}
+	}
 
 	d->f = fopen(fname, d->writable? "r+" : "r");
 	if (d->f == NULL) {
