@@ -2,7 +2,7 @@
 #define	CPU_H
 
 /*
- *  Copyright (C) 2005-2010  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2018  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -229,6 +229,7 @@ struct physpage_ranges {
 
 
 /*  Include all CPUs' header files here:  */
+#include "cpu_alpha.h"
 #include "cpu_arm.h"
 #include "cpu_m88k.h"
 #include "cpu_mips.h"
@@ -311,7 +312,7 @@ struct cpu_family {
 
 #define	MAX_DYNTRANS_READAHEAD		128
 
-#define	DEFAULT_DYNTRANS_CACHE_SIZE	(48*1048576)
+#define	DEFAULT_DYNTRANS_CACHE_SIZE	(96*1048576)
 #define	DYNTRANS_CACHE_MARGIN		200000
 
 #define	N_BASE_TABLE_ENTRIES		65536
@@ -436,6 +437,7 @@ struct cpu {
 	 *  specifics, etc.
 	 */
 	union {
+		struct alpha_cpu      alpha;
 		struct arm_cpu        arm;
 		struct m88k_cpu       m88k;
 		struct mips_cpu       mips;
