@@ -2,7 +2,7 @@
 #define	MACHINE_H
 
 /*
- *  Copyright (C) 2005-2010  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2018  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -73,7 +73,7 @@ struct tick_functions {
 	/*  Arrays, with one element for each entry:  */
 	int	*ticks_till_next;
 	int	*ticks_reset_value;
-	void	(*(*f))(struct cpu *, void *);
+	void	(**f)(struct cpu *, void *);
 	void	**extra;
 };
 
@@ -102,7 +102,7 @@ struct machine {
 	struct settings *settings;
 
 	/*  Name as choosen by the user:  */
-	char	*name;
+	const char *name;
 
 	/*  Full "path" to the machine, e.g. "machine[0]":  */
 	char	*path;
@@ -112,7 +112,7 @@ struct machine {
 	int	machine_subtype;	/*  MACHINE_DEC_3MAX_5000, ..  */
 
 	/*  Name set by code in src/machines/machine_*.c:  */
-	char	*machine_name;
+	const char *machine_name;
 
 	/*  The serial number is mostly used when emulating multiple machines
 	    in a network. nr_of_nics is the current nr of network cards, which
@@ -202,9 +202,10 @@ struct machine {
 #define	ARCH_NOARCH		0
 #define	ARCH_MIPS		1
 #define	ARCH_PPC		2
-#define	ARCH_ARM		3
-#define	ARCH_SH			4
-#define	ARCH_M88K		5
+#define	ARCH_ALPHA		4
+#define	ARCH_ARM		5
+#define	ARCH_SH			6
+#define	ARCH_M88K		7
 
 /*  MIPS:  */
 #define	MACHINE_BAREMIPS	1000
@@ -218,6 +219,7 @@ struct machine {
 #define	MACHINE_EVBMIPS		1008
 #define	MACHINE_ALGOR		1009
 #define	MACHINE_QEMU_MIPS	1010
+#define	MACHINE_VOCORE		1011
 
 /*  PPC:  */
 #define	MACHINE_BAREPPC		2000
@@ -227,6 +229,11 @@ struct machine {
 #define	MACHINE_MACPPC		2004
 #define	MACHINE_MVMEPPC		2005
 
+/*  Alpha:  */
+#define	MACHINE_BAREALPHA	4000
+#define	MACHINE_TESTALPHA	4001
+#define	MACHINE_ALPHA		4002
+
 /*  ARM:  */
 #define	MACHINE_BAREARM		5000
 #define	MACHINE_TESTARM		5001
@@ -236,6 +243,7 @@ struct machine {
 #define	MACHINE_IQ80321		5005
 #define	MACHINE_IYONIX		5006
 #define	MACHINE_QEMU_ARM	5007
+#define	MACHINE_RPI		5008
 
 /*  SH:  */
 #define	MACHINE_BARESH		6000
@@ -248,6 +256,7 @@ struct machine {
 #define	MACHINE_BAREM88K	7000
 #define	MACHINE_TESTM88K	7001
 #define	MACHINE_MVME88K		7002
+#define	MACHINE_LUNA88K		7003
 
 /*  Other "pseudo"-machines:  */
 #define	MACHINE_NONE		0
@@ -320,6 +329,10 @@ struct machine {
 #define	MACHINE_MVME88K_188		2
 #define	MACHINE_MVME88K_197		3
 
+/*  LUNA88K  */
+#define MACHINE_LUNA_88K		1
+#define MACHINE_LUNA_88K2		2
+
 
 /*  For the automachine system:  */
 struct machine_entry_subtype {
