@@ -128,8 +128,10 @@ void machine_destroy(struct machine *machine)
 	for (i=0; i<machine->ncpus; i++)
 		cpu_destroy(machine->cpus[i]);
 
-	if (machine->name != NULL)
-		free(machine->name);
+	// TODO: Memory leak; but it's ok, since the whole legacy thing should
+	// be replaced anyway.
+	// if (machine->name != NULL)
+	// 	free(machine->name);
 
 	if (machine->path != NULL)
 		free(machine->path);
@@ -441,7 +443,7 @@ void machine_dumpinfo(struct machine *m)
 /*
  *  machine_setup():
  *
- *  This (rather large) function initializes memory, registers, and/or devices
+ *  This function initializes memory, registers, and/or devices
  *  required by specific machine emulations.
  */
 void machine_setup(struct machine *machine)
@@ -699,7 +701,7 @@ void machine_entry_add_alias(struct machine_entry *me, const char *name)
 	CHECK_ALLOCATION(me->aliases = (char **) realloc(me->aliases,
 	    sizeof(char *) * me->n_aliases));
 
-	me->aliases[me->n_aliases - 1] = (char *) name;
+	me->aliases[me->n_aliases - 1] = strdup(name);
 }
 
 
@@ -820,7 +822,7 @@ void machine_list_available_types_and_cpus(void)
 		fatal("No machines defined!\n");
 
 	while (me != NULL) {
-		int i, j, iadd = DEBUG_INDENTATION;
+		int i, j, iadd2 = DEBUG_INDENTATION;
 
 		debug("%s [%s] (", me->name,
 		    cpu_family_ptr_by_number(me->arch)->name);
@@ -828,7 +830,7 @@ void machine_list_available_types_and_cpus(void)
 			debug("%s\"%s\"", i? ", " : "", me->aliases[i]);
 		debug(")\n");
 
-		debug_indentation(iadd);
+		debug_indentation(iadd2);
 		for (i=0; i<me->n_subtypes; i++) {
 			struct machine_entry_subtype *mes;
 			mes = me->subtype[i];
@@ -839,7 +841,7 @@ void machine_list_available_types_and_cpus(void)
 				    mes->aliases[j]);
 			debug(")\n");
 		}
-		debug_indentation(-iadd);
+		debug_indentation(-iadd2);
 
 		me = me->next;
 	}
